@@ -36,7 +36,7 @@ class DataBase(metaclass=Singleton):
     _systems = Systems()
 
     # write methods
-    write_weather_data = _write.write_weather_data
+    write_weather_data = _write.write_weather_forecast_data
     write_realised = _write.write_realised
     write_realised_pvdata = _write.write_realised_pvdata
     write_kpi = _write.write_kpi
@@ -64,7 +64,7 @@ class DataBase(metaclass=Singleton):
     )
     # predictor methods
     get_predictors = _predictor.get_predictors
-    get_electricity_price = _predictor.get_electricity_price
+    get_electricity_price = _predictor.get_day_ahead_electricity_price
     get_load_profiles = _predictor.get_load_profiles
     # historic cdb data service
     get_load_sid = _historic_cdb_data_service.get_load_sid
@@ -113,13 +113,14 @@ class DataBase(metaclass=Singleton):
                 influxdb_token (str): Token to authenticate to InfluxDB.
                 influxdb_host (str): InfluxDB host.
                 influxdb_port (int): InfluxDB port.
-                mysql_username (str): MySQL username.
-                mysql_password (str): MySQL password.
-                mysql_host (str): MySQL host.
-                mysql_port (int): MYSQL port.
-                mysql_database_name (str): MySQL database name.
+                influx_organization (str): InfluxDB organization.
+                sql_db_username (str): SQL database username.
+                sql_db_password (str): SQL database password.
+                sql_db_host (str): SQL database host.
+                sql_db_port (int): SQL database port.
+                sql_db_database_name (str): SQL database name.
                 proxies Union[dict[str, str], None]: Proxies.
-
+                sql_db_type (str, optional): SQL Database type ('mysql' or 'postgresql').
         """
 
         self._datainterface = _DataInterface(config)
