@@ -3,7 +3,7 @@
 # SPDX-License-Identifier: MPL-2.0
 
 from datetime import datetime, timedelta
-from typing import Tuple, Union
+from typing import Tuple, Union, List
 
 import numpy as np
 import pandas as pd
@@ -25,9 +25,12 @@ class ModelInput:
         self,
         pid: int = 295,
         location: Union[Tuple[int, int], str] = "Arnhem",
+        country: str = "NL",
         datetime_start: str = None,
         datetime_end: str = None,
         forecast_resolution: str = "15min",
+        entsoe_bidding_zone: str = "NL",
+        weather_source: Union[List[str], str] = "optimum",
     ) -> pd.DataFrame:
         """Get model input.
 
@@ -38,11 +41,14 @@ class ModelInput:
         Args:
             pid (int, optional): Prediction job id. Defaults to 295.
             location (str, optional): Location name or tuple with lat, lon. Defaults to "Arnhem".
+            country (str, optional): Country of were the location is located. Defaults to "NL", The Netherlands.
             datetime_start (datetime, optional): Start datetime. Defaults to None.
             datetime_end (datetime, optional): End datetime. Defaults to None.
             forecast_resolution (str, optional): Time resolution of model input
                 (see pandas Date Offset frequency strings). Defaults to "15min".
-
+            entsoe_bidding_zone (str, optional): Name of the market place if market data is requested.
+                Default to "NL".
+            weather_source (str or list of str, optional): which weather models should be used in preferred order.
         Returns:
             pd.DataFrame: Model input.
         """
@@ -77,6 +83,9 @@ class ModelInput:
             datetime_end=datetime_end,
             forecast_resolution=forecast_resolution,
             location=location,
+            country=country,
+            entsoe_bidding_zone=entsoe_bidding_zone,
+            source=weather_source,
         )
 
         # Create model input with datetime index
@@ -90,6 +99,7 @@ class ModelInput:
         else:
             self.logger.warning("No load data returned, fill with NaN.")
             model_input["load"] = np.nan
+
         # Add predictors
         model_input = pd.concat([model_input, predictors], axis=1)
 
@@ -104,6 +114,8 @@ class ModelInput:
         history: int = 14,
         datetime_start: datetime = None,
         sid: str = None,
+        country: str = "NL",
+        source: str = "optimum",
     ) -> pd.DataFrame:
         """This function retrieves the radiation and cloud forecast for the nearest weather location
         and the relevant pvdata from a specific system or region.
@@ -117,6 +129,11 @@ class ModelInput:
             - forecastResolution: time resolution of forecast in minutes [int]
             - datetime_start: datetime of forecast
             - source: preferred weather source as a string, default for wind is DSN
+            - country (str, optional): Country of were the location is located. Defaults to "NL", The Netherlands.
+            - source (str or list of str): which weather models should be used.
+                Options: "OWM", "DSN", "WUN", "harmonie", "harm_arome", "harm_arome_fallback", "icon", "optimum",
+                Default: 'optimum'. This combines harmonie, harm_arome, icon and DSN,
+                taking the (heuristicly) best available source for each moment in time
         """
         if datetime_start is None:
             datetime_start = datetime.utcnow()
@@ -145,11 +162,11 @@ class ModelInput:
         end = datetime_start + timedelta(minutes=forecast_horizon)
 
         weather_data = Weather().get_weather_data(
-            location, weather_params, start, end, source="optimum"
+            location, weather_params, start, end, source, forecast_resolution, country
         )
 
         # Interpolate weather data to 15 minute values
-        weather_data = weather_data.resample(str(forecast_resolution) + "T").asfreq()
+        weather_data = weather_data.resample(str(forecast_resolution) + "min").asfreq()
         for col in weather_params:
             if col in weather_data:
                 weather_data.loc[:, col] = weather_data.loc[:, col].interpolate(
@@ -159,7 +176,12 @@ class ModelInput:
         # Get PV_load from influx (end time at start of forecast)
         end = datetime_start
         pvdata = Ems().get_load_sid(
-            sid, start, end, "15T", aggregated=True, average_output=radius == 0
+            sid,
+            start,
+            end,
+            forecast_resolution,
+            aggregated=True,
+            average_output=radius == 0,
         )
 
         # If no load was found return None
@@ -193,6 +215,7 @@ class ModelInput:
         forecast_resolution: int,
         datetime_start: datetime = None,
         source: str = "optimum",
+        country: str = "NL",
     ) -> pd.DataFrame:
         """This function retrieves the wind speed forecast for the nearest weather location
         and calculates the wind speed based on the turbine's hub height.
@@ -204,6 +227,7 @@ class ModelInput:
             forecast_resolution: time resolution of forecast in minutes [int]
             datetime_start: datetime of forecast
             source: preferred weather source as a string, default for wind is DSN
+            country (str, optional): Country of were the location is located. Defaults to "NL", The Netherlands.
         """
 
         if datetime_start is None:
@@ -216,10 +240,12 @@ class ModelInput:
             datetime_start=datetime_start,
             datetime_end=datetime_end,
             source=source,
+            resolution=forecast_resolution,
+            country=country,
         )
 
         # interpolate results to 15 minute values
-        windspeed = windspeed.resample(str(forecast_resolution) + "T").asfreq()
+        windspeed = windspeed.resample(str(forecast_resolution) + "min").asfreq()
         windspeed = windspeed.interpolate("cubic")
 
         return pd.DataFrame(windspeed.windspeed_100m)
@@ -229,7 +255,7 @@ class ModelInput:
         """ "This function retrieves the power curve coefficients from the genericpowercurves table,
         using the turbine type as input."""
         bind_params = {"turbine_type": turbine_type}
-        query = "SELECT * FROM genericpowercurves WHERE name = %(turbine_type)s"
+        query = "SELECT * FROM genericpowercurves WHERE name = :turbine_type"
 
         result = _DataInterface.get_instance().exec_sql_query(query, bind_params)
 
