@@ -9,7 +9,7 @@ import pandas as pd
 import re
 
 from openstef_dbc.data_interface import _DataInterface
-from openstef_dbc.utils import parse_influx_result
+from openstef_dbc.utils import floor_to_closest_time_resolution, parse_influx_result
 
 FIELDS_OF_INTEREST = [
     "forecast",
@@ -33,8 +33,8 @@ class Predictions:
 
         Args:
             pj (dict): Prediction job
-            start_time (datetime): Start time  to retrieve the historic load prediction.
-            end_time (datetime): End timeto retrieve the historic load prediction.
+            start_time (datetime): Start time to retrieve the historic load prediction.
+            end_time (datetime): End time to retrieve the historic load prediction.
 
         Returns:
             pandas.Series: Forecast column with the predicted values
@@ -45,6 +45,13 @@ class Predictions:
             start_time = datetime.utcnow()
         if end_time is None:
             end_time = datetime.utcnow() + timedelta(days=2)
+        else:
+            end_time = end_time + timedelta(
+                minutes=15
+            )  # To make the ouput the same as the original influx client.
+
+        # Round end_time to last resolution_minutes interval
+        end_time = floor_to_closest_time_resolution(end_time, pj["resolution_minutes"])
 
         bind_params = {
             "pid": str(pj["id"]),
@@ -64,7 +71,6 @@ class Predictions:
                 |> filter(fn: (r) => r.pid == "{bind_params["pid"]}")
                 |> aggregateWindow(every: {pj["resolution_minutes"]}m, fn: mean)
         """
-
         # Query the database
         result = _DataInterface.get_instance().exec_influx_query(query, bind_params)
 
@@ -75,7 +81,7 @@ class Predictions:
         # Return result
         if not result.empty:
             # Shifting is needed to output the same as with the old influx client
-            return parse_influx_result(result).shift(-15, freq="T")
+            return parse_influx_result(result).shift(-15, freq="min")
         else:
             return pd.Series()
 
@@ -123,8 +129,7 @@ class Predictions:
 
         # Return result
         if not result.empty:
-            # Shifting is needed to output the same as with the old influx client
-            return parse_influx_result(result, aggfunc="first").shift(-15, freq="T")
+            return parse_influx_result(result, aggfunc="first")
         else:
             return pd.Series()
 
@@ -204,7 +209,7 @@ class Predictions:
         # Return result
         if not result.empty:
             # Shifting is needed to output the same as with the old influx client
-            return parse_influx_result(result).shift(-15, freq="T")
+            return parse_influx_result(result).shift(-15, freq="min")
         else:
             return pd.Series()
 
@@ -334,7 +339,7 @@ class Predictions:
         if not result.empty:
             # Shifting is needed to output the same as with the old influx client
             result = parse_influx_result(result, aditional_indices=["tAhead"]).shift(
-                -15, freq="T"
+                -15, freq="min"
             )
         else:
             return pd.Series()
