@@ -15,6 +15,7 @@ from influxdb_client.client.warnings import MissingPivotFunction
 
 from openstef_dbc.data_interface import _DataInterface
 from openstef_dbc.services.write import Write
+from openstef_dbc.settings import Settings
 from openstef_dbc.utils import genereate_datetime_index, parse_influx_result
 
 warnings.simplefilter("ignore", MissingPivotFunction)
@@ -42,7 +43,7 @@ class Weather:
         query = """
             SELECT input_city as city, lat, lon, country
             FROM weatherforecastlocations
-            WHERE country = %(country)s AND active = %(active)s
+            WHERE country = :country AND active = :active
         """
         result = _DataInterface.get_instance().exec_sql_query(query, bind_params)
 
@@ -50,12 +51,16 @@ class Weather:
 
         return locations
 
-    def _get_nearest_weather_location(
-        self, location: Union[Tuple[float, float], str], threshold: float = 150.0
-    ) -> str:
-        """Find the nearest weather forecast location.
+    def _get_nearest_weather_locations(
+        self,
+        location: Union[Tuple[float, float], str],
+        country: str = "NL",
+        threshold: float = 150.0,
+        number_locations: int = 1,
+    ) -> pd.Series:
+        """Find the nearest weather forecast locations.
 
-        Function that, given an location, finds the nearest location for which a
+        Function that, given an location, finds the nearest locations for which a
         weatherforecast is available. A warning is generated when the distance is
         greater than a certain distance (threshold).
 
@@ -63,13 +68,17 @@ class Weather:
 
         Args:
             location (str, tuple): Name of the location/city or coordinates (lat, lon).
+            country (str): Country code (2-letter: ISO 3166-1).
             threshold (int): Maximum distance [km] before a warning is generated.
+            number_locations (int): number of weather locations desired
 
         Returns:
-            str: The name of the weather forecast location.
+            Pd.Series: Name of the weather location(s) (distance as index)
         """
         # Get all available cities
-        weather_locations = self.get_weather_forecast_locations(country="NL", active=1)
+        weather_locations = self.get_weather_forecast_locations(
+            country=country, active=1
+        )
 
         # If location is string, convert to (lat, lon)
         if type(location) is str:
@@ -78,7 +87,7 @@ class Weather:
             location_coordinates = location
 
         # Now we have the coordinates of the input_city. Next, find nearest weather location
-        distances = pd.DataFrame(columns=["distance", "input_city"])
+        distance_dfs = []
         for weather_location in weather_locations:
             coordinates = (weather_location["lat"], weather_location["lon"])
 
@@ -89,22 +98,19 @@ class Weather:
                 )
 
             distance = round(
-                geopy.distance.geodesic(coordinates, location_coordinates).km
+                geopy.distance.geodesic(coordinates, location_coordinates).km, 1
             )
             city = weather_location["city"]
             distance_df = pd.DataFrame([{"distance": distance, "input_city": city}])
-            distances = pd.concat([distances, distance_df])
+            distance_dfs.append(distance_df)
 
-        distances = distances.set_index("distance")
+        distances = pd.concat(distance_dfs).set_index("distance")
 
-        nearest_location = distances["input_city"][distances.index.min()]
+        nearest_location = distances["input_city"].sort_index().iloc[0:number_locations]
 
         # Find closest weather location
         if distances.index.min() < threshold:
-            if isinstance(nearest_location, pd.Series):
-                return nearest_location.reset_index(drop=True)[0]
-            else:
-                return nearest_location
+            return nearest_location
 
         raise Warning(
             "Closest weather location is farther than threshold: {dist} > {threshold}".format(
@@ -130,7 +136,7 @@ class Weather:
 
         # Query corresponding (lat, lon) from SQL database
         binding_params = {"city": location_name}
-        query = "SELECT lat, lon from NameToLatLon where regionInput = %(city)s"
+        query = "SELECT lat, lon from NameToLatLon where regionInput = :city"
         location = _DataInterface.get_instance().exec_sql_query(query, binding_params)
 
         # If not found
@@ -148,6 +154,31 @@ class Weather:
 
         return location
 
+    def _get_source_run(
+        self, forecast_datetime: pd.Series, tAhead: pd.Series
+    ) -> pd.Series:
+        """Compute the datetime when weather forecast was created
+
+        Args:
+            forecast_datetime (pd.Series[datetime]): forecasted datetime.
+            tAhead (pd.Series[(int, float)]: forecasting horizon in hours
+
+        Retuns :
+            pd.Series of new datetimes
+        """
+
+        if not pd.api.types.is_datetime64_any_dtype(forecast_datetime):
+            raise ValueError("forecast_datetime must be a Series of datetime.")
+
+        if not pd.api.types.is_numeric_dtype(tAhead):
+            raise ValueError("tahead must be a Series of int or float.")
+
+        if len(forecast_datetime) != len(tAhead):
+            raise ValueError("forecast_datetime and tAhead must have the same length.")
+
+        # Compute new datetimes
+        return forecast_datetime - pd.to_timedelta(tAhead, unit="h")
+
     def _combine_weather_sources(
         self, result: pd.DataFrame, source_order: List = None
     ) -> pd.DataFrame:
@@ -159,8 +190,9 @@ class Weather:
         Args:
             result (pd.DataFrame): return value from the function 'get_weatherdata'
             source_order (list): which weather models should be used (first).
-                Options: "OWM", "DSN", "WUN", "harmonie", "harm_arome", "optimum"
-                Default: '"harm_arome", "harmonie", "DSN"'. This combines harmonie, harm_arome and DSN,
+                Options: "OWM", "DSN", "WUN", "harmonie", "harm_arome", "harm_arome_fallback", "optimum", "icon"
+                Default: '"harm_arome", "harm_arome_fallback", "harmonie", "icon", "DSN"'.
+                This combines harmonie, harm_arome, harm_arome_fallback, ICON and DSN,
                 taking the (heuristicly) best available source for each moment in time
 
         Returns:
@@ -177,7 +209,7 @@ class Weather:
         # step 1: Create list of multiple dataframes,
         # check which of the 'optimum' sources are actually in the list
         if source_order is None:
-            source_order = ["harm_arome", "GFS_50", "harmonie", "DSN"]
+            source_order = Settings.optimum_weather_sources
 
         active_sources = [s for s in source_order if s in set(result.source)]
         # for each source a seperate dataframe
@@ -196,10 +228,13 @@ class Weather:
         self,
         location: Union[Tuple[float, float], str],
         weatherparams: List[str],
-        datetime_start: datetime = None,
-        datetime_end: datetime = None,
+        datetime_start: datetime = datetime.utcnow() - timedelta(days=14),
+        datetime_end: datetime = datetime.utcnow() + timedelta(days=2),
         source: Union[List[str], str] = "optimum",
         resolution: str = "15min",
+        country: str = "NL",
+        number_locations: int = 1,
+        type: str = "smallest_tAhead",
     ) -> pd.DataFrame:
         """Get weather data from database.
 
@@ -215,10 +250,13 @@ class Weather:
             datetime_start (datetime) : start date time. Default: 14 days ago
             datetime_end (datetime): end date time. Default: now + 2 days.
             source (str or list of str): which weather models should be used.
-                Options: "OWM", "DSN", "WUN", "harmonie", "harm_arome", "optimum"
-                Default: 'optimum'. This combines harmonie, harm_arome and DSN,
+                Options: "OWM", "DSN", "WUN", "harmonie", "harm_arome", "harm_arome_fallback", "icon", "optimum",
+                Default: 'optimum'. This combines harmonie, harm_arome, icon and DSN,
                 taking the (heuristicly) best available source for each moment in time
-            resolution (str): Time resolution of the returned data, default: "15T"
+            resolution (str): Time resolution of the returned data, default: "15min"
+            country (str): Country code (2-letter: ISO 3166-1). e.g. NL
+            number_locations (int): number of weather locations desired
+            type (str) : type of weather forecast (smallest_tAhead of multiple_tAheads)
         Returns:
             pd.DataFrame: The most recent weather prediction
 
@@ -229,14 +267,8 @@ class Weather:
             print(df.head())
         """
 
-        if datetime_start is None:
-            datetime_start = datetime.utcnow() - timedelta(days=14)
-
+        # Converting to datetime objet with tz attribute
         datetime_start = pd.to_datetime(datetime_start)
-
-        if datetime_end is None:
-            datetime_end = datetime.utcnow() + timedelta(days=2)
-
         datetime_end = pd.to_datetime(datetime_end)
 
         # Convert to UTC and remove UTC as note
@@ -246,12 +278,9 @@ class Weather:
         if datetime_end.tz is not None:
             datetime_end = datetime_end.tz_convert("UTC").tz_localize(None)
 
-        # Get data from an hour earlier to correct for radiation shift later
-        datetime_start_original = datetime_start.tz_localize("UTC")
-
-        datetime_start -= timedelta(hours=1)
-
-        location_name = self._get_nearest_weather_location(location)
+        location_name = self._get_nearest_weather_locations(
+            location=location, country=country, number_locations=number_locations
+        )
 
         # Make a list of the source and weatherparams.
         # Like this, it also works if source is a string instead of multiple values
@@ -263,14 +292,13 @@ class Weather:
         # Try to get the data from influx.
         if "optimum" in source:
             # so the query return all
-            source = ["harm_arome", "GFS_50", "harmonie", "DSN"]
+            source = Settings.optimum_weather_sources
             combine_sources = True
         else:
             combine_sources = False
 
         # Initialize binding params
         bind_params = {
-            "_input_city": location_name,
             "_start": datetime_start,
             "_stop": datetime_end,
         }
@@ -278,12 +306,24 @@ class Weather:
         # Initialise strings for the querying influx, it is not possible to parameterize this string
         weather_params_str = '" or r._field == "'.join(weatherparams)
         weather_models_str = '" or r.source == "'.join(source)
+        weather_location_name_str = '" or r.input_city == "'.join(
+            location_name.to_list()
+        )
+
+        if type == "smallest_tAhead":
+            weather_measurement_str = "weather"
+            influx_indices = ["source", "input_city"]
+            grouping_indices = ["source", "input_city"]
+        elif type == "multiple_tAheads":
+            weather_measurement_str = "weather_tAhead"
+            influx_indices = ["source", "input_city", "tAhead"]
+            grouping_indices = ["source", "input_city", "created"]
 
         # Create the query
         query = f"""
             from(bucket: "forecast_latest/autogen") 
                 |> range(start: {bind_params["_start"].strftime('%Y-%m-%dT%H:%M:%SZ')}, stop: {bind_params["_stop"].strftime('%Y-%m-%dT%H:%M:%SZ')}) 
-                |> filter(fn: (r) => r._measurement == "weather" and (r._field == "{weather_params_str}") and (r.source == "{weather_models_str}") and r.input_city == "{bind_params["_input_city"]}")
+                |> filter(fn: (r) => r._measurement == "{weather_measurement_str}" and (r._field == "{weather_params_str}") and (r.source == "{weather_models_str}") and (r.input_city == "{weather_location_name_str}"))
         """
 
         # Execute Query
@@ -291,11 +331,13 @@ class Weather:
 
         # For multiple Fields a list is returned.
         if isinstance(result, list):
-            result = pd.concat(result)[["_value", "_field", "_time", "source"]]
+            result = pd.concat(result)[
+                ["_value", "_field", "_time", "source", "input_city"]
+            ]
 
         # Check if response is empty
         if not result.empty:
-            result = parse_influx_result(result, ["source"])
+            result = parse_influx_result(result, influx_indices)
         else:
             self.logger.warning("No weatherdata found. Returning empty dataframe")
             return pd.DataFrame(
@@ -310,27 +352,41 @@ class Weather:
         if combine_sources:
             self.logger.info("Combining sources into single dataframe")
             result = self._combine_weather_sources(result)
+            result["source"] = "optimum"
 
-        # Interpolate if nescesarry
-        result = result.resample(resolution).interpolate(limit=11)
+        # Compute source_run
+        if type == "multiple_tAheads":
+            result["created"] = self._get_source_run(result.index, result.tAhead)
 
-        # Shift radiation by 30 minutes if resolution allows it
-        if "radiation" in result.columns:
-            shift_delta = -timedelta(minutes=30)
-            if shift_delta % pd.Timedelta(resolution) == timedelta(0):
-                result["radiation"] = result["radiation"].shift(1, shift_delta)
+        # Interpolate if nescesarry by input_city, source (and tAhead)
+        with pd.option_context("future.no_silent_downcasting", True):
+            result = (
+                result.groupby(grouping_indices)
+                .resample(resolution, include_groups=False)
+                .interpolate(limit=11)
+                .reset_index(grouping_indices)
+            )
 
-        # Drop extra rows not neccesary
-        result = result.loc[datetime_start_original:]
+        if number_locations == 1:
+            result = result.drop(columns="input_city")
+        else:
+            # Adding distance information en km
+            result = (
+                result.reset_index()
+                .merge(location_name.reset_index(), how="left", on="input_city")
+                .set_index("datetime")
+            )
 
         return result
 
-    def get_datetime_last_stored_knmi_weatherdata(self) -> datetime:
-        query = """
+    def get_datetime_last_stored_knmi_weatherdata(
+        self, source: str = "harm_arome"
+    ) -> datetime:
+        """Returns datetime of latest KNMI run in influx Database. If no run is found return first unix time."""
+        query = f"""
             from(bucket: "forecast_latest/autogen" )   
-                |> range(start: - 10d) 
-                |> limit(n:10)
-                |> filter(fn: (r) => r._measurement == "weather" and r.source == "harm_arome" and r._field == "source_run")
+                |> range(start: - 2d)
+                |> filter(fn: (r) => r._measurement == "weather" and r.source == "{source}" and r._field == "source_run")
                 |> max()
         """
         result = _DataInterface.get_instance().exec_influx_query(query)
@@ -338,6 +394,8 @@ class Weather:
         if isinstance(result, pd.DataFrame) and not result.empty:
             # Get latest run
             latest_unix = result["_value"].max()
+        else:
+            latest_unix = 0  # Return first unix time if no run is found
 
         # unix timestamp of most recent stored weather forecast created
         last_stored_run = datetime.fromtimestamp(latest_unix, pytz.utc)
