@@ -2,15 +2,17 @@
 #
 # SPDX-License-Identifier: MPL-2.0
 
-from typing import Tuple, List
+from typing import Tuple, List, Dict
 from datetime import datetime
 import time
+import re
 
 import numpy as np
 import pandas as pd
 
 from openstef_dbc.data_interface import _DataInterface
 from openstef_dbc.log import logging
+from openstef_dbc.utils import round_down_time_differences
 
 
 class Write:
@@ -19,13 +21,14 @@ class Write:
 
     def write_location(self, location_name: str, location: Tuple[float, float]) -> None:
         bind_params = {
-            "table_name": "NameToLatLon",
             "loc": location_name,
             "lat": location[0],
             "lon": location[1],
         }
 
-        statement = "INSERT INTO %(table_name)s (regionInput, lat,lon) VALUES (%(loc)s, %(lat)s, %(lon)s)"
+        statement = (
+            "INSERT INTO NameToLatLon (regionInput, lat,lon) VALUES (:loc, :lat, :lon)"
+        )
 
         _DataInterface.get_instance().exec_sql_write(statement, params=bind_params)
 
@@ -67,14 +70,14 @@ class Write:
 
         forecast = data.copy()
         # Add created to data
-        forecast["created"] = int(datetime.utcnow().timestamp())
+        forecast["created"] = str(int(datetime.utcnow().timestamp()))
 
         # Add quality column. Fill nan's or missing column with 'actual'
         if force_quality:
             if "quality" not in forecast.columns:
                 forecast["quality"] = "actual"
             else:
-                forecast[forecast["quality"].isna()] = "actual"
+                forecast.loc[forecast["quality"].isna(), "quality"] = "actual"
 
         # Write DataFrame to influx database
         # Find tag columns
@@ -83,10 +86,42 @@ class Write:
         field_columns = [x for x in forecast.columns if x not in tag_columns]
 
         # Cast columns to correct type, as influx is extremely picky
-        casting_dict = {"prediction": float, "stdev": float}
-        for col, cast in casting_dict.items():
-            if col in field_columns:
-                forecast = forecast.astype({col: cast})
+        casting_dict = {
+            "prediction": np.float64,
+            "stdev": np.float64,
+            "pid": np.int64,
+            "type": str,
+            "customer": str,
+            "created": str,
+            "algtype": str,
+            "description": str,
+            "forecast_solar": np.float64,
+            "forecast_wind_on_shore": np.float64,
+            "forecast_other": np.float64,
+            "forecast": np.float64,
+            "quality": str,
+            "tAhead": np.float64,
+        }
+        # Generate casting dict for available quantiles
+        p = re.compile(r"quantile_P\d\d")
+        quantile_columns = [s for s in field_columns if p.match(s)]
+        casting_dict.update(dict.fromkeys(quantile_columns, np.float64))
+
+        # Check if we have all columns and not some extra
+        casting_dict_columns = list(casting_dict.keys())
+        for k in casting_dict_columns:
+            # Remove any casting dict entries that are not in the dataframe
+            if k not in forecast.columns:
+                casting_dict.pop(k, None)
+
+        if set(casting_dict.keys()) != set(
+            forecast.columns.to_list()
+        ):  # Check if we have a type description for every column, if not raise an error
+            raise ValueError(
+                "Got unexpected columns, unable to cast these columns in the correct datatype."
+            )
+
+        forecast = forecast.astype(casting_dict)
 
         result = _DataInterface.get_instance().exec_influx_write(
             forecast.copy(),
@@ -141,20 +176,17 @@ class Write:
         influxtable = "prediction_tAheads"
 
         # specify desired t_aheads
-        desired_t_aheads = [0.0, 1.0, 4.0, 8.0, 24.0, 47.0, 50.0, 144.0]
+        desired_t_aheads = [0.0, 1.0, 4.0, 8.0, 15.0, 24.0, 39.0, 47.0, 50.0, 144.0]
 
         t_adf = forecast.copy()
 
-        t_adf["tAhead"] = np.floor(
-            (t_adf.index.tz_localize(None) - datetime.utcnow()).total_seconds() / 3600
-        )
-        # floor all tAheads to first tAhead lower than or equal to calculated tAhead
-        # 1000 as a fill value if all else fails
-        # !TODO check this!
-        t_adf["tAhead"] = [
-            min([1000.0] + [float(x) for x in desired_t_aheads if x >= t_ahead])
-            for t_ahead in t_adf["tAhead"]
-        ]
+        # Calculate tAheads
+        timediffs = (
+            t_adf.index.tz_localize(None) - datetime.utcnow()
+        ).total_seconds() / 3600
+        # Round it to the first bigger desired_t_ahead
+        t_adf["tAhead"] = round_down_time_differences(timediffs, desired_t_aheads)
+
         t_adf = t_adf.loc[
             [x in desired_t_aheads for x in t_adf.tAhead],
             [x for x in allowed_columns if x in t_adf.columns],
@@ -193,50 +225,43 @@ class Write:
         )
         return f"Written {num_rows} rows to {dbname}.{influxtable}"
 
-    def write_weather_data(
+    def _write_weather_forecast_data_to_influx(
         self,
-        data: pd.DataFrame,
-        source: str,
-        table: str = "weather",
-        dbname: str = "forecast_latest",
-        tag_columns: List[str] = None,
+        influx_df: pd.DataFrame,
+        dbname: str,
+        table: str,
+        tag_columns: List[str],
+        field_columns: List[str],
+        casting_dict: Dict[str, type],
     ):
-        """This function should be used to write data directly to our database.
-        Do not use this function for Measurements data (pvdata, demanddata, winddata).
-        Using the PostData function is prefered for this data, since traffic monitoring is ensured.
+        """Writes weather forecasts to influx database after casting the columns to the correct datatype.
 
-        Parameters:
-            - data: pd.DataFrame(index = "datetimeFC", columns = ['input_city','temp','windspeed'])
-            - source: str: source of the weatherdata. Obligated!
-            - table: str
-            - insertIgnore: Bool
-            - tag_columns: list: the column names used as tags in influx
+        Args:
+            influx_df: pd.DataFrame(index = "datetimeFC", columns = ['input_city','temp','windspeed'])
+            dbname: (str) database name
+            table: (str) table name
+            tag_columns: (list) the column names used as tags in influx
+            field_columns: (list) the column names used as fields in influx
+            casting_dict: (dict) dictionary with column names as keys and the desired datatype as values
 
-        Output:
-            message: string
+        Returns:
+            None
         """
+        # Check if we have all columns and not some extra
+        casting_dict_columns = list(casting_dict.keys())
+        for k in casting_dict_columns:
+            # Remove any casting dict entries that are not in the dataframe
+            if k not in influx_df.columns:
+                casting_dict.pop(k, None)
 
-        # These names are forbidden to be used as fields, as thy will break everything!
-        # Influx is very picky about fields vs tags
-        if tag_columns is None:
-            tag_columns = ["input_city", "source"]
+        if set(casting_dict.keys()) != set(
+            influx_df.columns.to_list()
+        ):  # Check if we have a type description for every column, if not raise an error
+            raise ValueError(
+                "Got unexpected columns, unable to cast these columns in the correct datatype."
+            )
 
-        forbidden_fields = ["input_city", "source"]
-
-        message = ""  # Create string placeholder for return message
-
-        self.logger.info("Write weather data to database")
-
-        influx_df = data.copy()
-        influx_df["source"] = source
-        # Add created to data
-        influx_df["created"] = int(datetime.utcnow().timestamp())
-
-        field_columns = [
-            x
-            for x in list(influx_df.columns)
-            if x not in tag_columns + forbidden_fields
-        ]
+        influx_df = influx_df.astype(casting_dict)
 
         # Write DataFrame to influx database
         result = _DataInterface.get_instance().exec_influx_write(
@@ -249,15 +274,213 @@ class Write:
         )
 
         if result:
-            message += "Written " + str(len(influx_df)) + " rows to influx"
             self.logger.info(
                 "Succesfully written fields to database", field_columns=field_columns
             )
             self.logger.info(
                 "Successfully written tags to database", tag_columns=tag_columns
             )
+            self.logger.info("Written " + str(len(influx_df)) + " rows to influx")
+        else:
+            self.logger.error(
+                "Could not write weather data to database",
+                field_columns=field_columns,
+                tag_columns=tag_columns,
+            )
 
-        return message
+    def _write_weather_forecast_data_latest(
+        self,
+        data: pd.DataFrame,
+        source: str,
+        forecast_created_time: datetime,
+        table: str,
+        dbname: str,
+        tag_columns: List[str] = None,
+        casting_dict: Dict[str, type] = {},
+    ):
+        """Writes weather data to the database. This function writes the data to a table containing the latest forecasts.
+
+        Args:
+            data: pd.DataFrame(index = "datetimeFC", columns = ['input_city','temp','windspeed'])
+            source: (str) source of the weatherdata
+            forecast_created_time: (datetime) the time at which the forecast was created
+            table: (str) table name
+            dbname: (str) database name
+            tag_columns: (list) the column names used as tags in influx
+            casting_dict: (dict) dictionary with column names as keys and the desired datatype as values
+
+        Returns:
+            None
+        """
+
+        influx_df = data.copy()
+        influx_df["source"] = source
+        # Add created to data
+        influx_df["created"] = int(forecast_created_time.timestamp())
+
+        if tag_columns is None:
+            tag_columns = ["input_city", "source"]
+
+        field_columns = [x for x in list(influx_df.columns) if x not in tag_columns]
+
+        self._write_weather_forecast_data_to_influx(
+            influx_df,
+            dbname,
+            table,
+            tag_columns,
+            field_columns,
+            casting_dict,
+        )
+
+    def _write_weather_forecast_data_t_ahead(
+        self,
+        data: pd.DataFrame,
+        source: str,
+        forecast_created_time: datetime,
+        table: str,
+        dbname: str,
+        desired_t_aheads: List[float],
+        tag_columns: List[str] = None,
+        casting_dict: Dict[str, type] = {},
+    ):
+        """Write weather data to the database. This function writes the data to a table containing with the forecasts for different t_ahead values.
+
+        Args:
+            data: pd.DataFrame(index = "datetimeFC", columns = ['input_city','temp','windspeed'])
+            source: (str) source of the weatherdata
+            forecast_created_time: (datetime) the time at which the forecast was created
+            table: (str) table name
+            dbname: (str) database name
+            desired_t_aheads: (list) the t_ahead values for which the data should be written
+            tag_columns: (list) the column names used as tags in influx
+            casting_dict: (dict) dictionary with column names as keys and the desired datatype as values
+
+        Returns:
+            None
+        """
+
+        influx_df = data.copy()
+        influx_df["source"] = source
+        # Add created to data
+        influx_df["created"] = int(forecast_created_time.timestamp())
+
+        if tag_columns is None:
+            tag_columns = ["input_city", "source"]
+        tag_columns.append("tAhead")
+
+        # Calculate tAheads
+        timediffs = (
+            influx_df.index.tz_localize(None)
+            - forecast_created_time.replace(tzinfo=None)
+        ).total_seconds() / 3600
+        # Round it to the first bigger desired_t_ahead
+        influx_df["tAhead"] = round_down_time_differences(timediffs, desired_t_aheads)
+
+        # Remove the undesired t_aheads
+        influx_df = influx_df.loc[
+            [x in desired_t_aheads for x in influx_df.tAhead],
+            [x for x in influx_df.columns],
+        ]
+        # Ignore tag columns when determining field columns
+        field_columns = [x for x in influx_df.columns if x not in tag_columns]
+
+        # Set correct casting type for tAhead column
+        casting_dict["tAhead"] = np.float64
+
+        self._write_weather_forecast_data_to_influx(
+            influx_df,
+            dbname,
+            table,
+            tag_columns,
+            field_columns,
+            casting_dict,
+        )
+
+    def write_weather_forecast_data(
+        self,
+        data: pd.DataFrame,
+        source: str,
+        table: str = "weather",
+        dbname: str = "forecast_latest",
+        tag_columns: List[str] = None,
+        forecast_created_time: datetime = datetime.utcnow(),
+        desired_t_aheads: List[float] = [
+            1.0,
+            12.0,
+            15.0,
+            24.0,
+            36.0,
+            39.0,
+            48.0,
+            4 * 24.0,
+            6 * 24.0,
+        ],
+    ):
+        """Write weather forecast data to the database.
+        This function writes the data both to a table containing the latest forecasts,
+        and a similar table with suffix _tAheads with the forecasts for different t_ahead values.
+
+        Args:
+            data: pd.DataFrame(index = "datetimeFC", columns = ['input_city','temp','windspeed'])
+            source: (str) source of the weatherdata
+            table: (str) table name
+            dbname: (str) database name
+            tag_columns: (list) the column names used as tags in influx
+            forecast_created_time: (datetime) the time at which the forecast was created
+            desired_t_aheads: (list) the t_ahead values for which the data should be written
+
+        Returns:
+            None
+        """
+
+        # Cast columns to correct type, as influx is extremely picky
+        casting_dict = {
+            "source_run": np.int64,
+            "input_city": str,
+            "temp": np.float64,
+            "windspeed_100m": np.float64,
+            "windspeed": np.float64,
+            "winddeg": np.float64,
+            "clouds": np.float64,
+            "mxlD": np.float64,
+            "snowDepth": np.float64,
+            "pressure": np.float64,
+            "humidity": np.float64,
+            "clearSky_ulf": np.float64,
+            "clearSky_dlf": np.float64,
+            "radiation": np.float64,
+            "radiation_direct": np.float64,
+            "radiation_diffuse": np.float64,
+            "radiation_normal": np.float64,
+            "windspeed_100m_ensemble": np.float64,
+            "windspeed_ensemble": np.float64,
+            "winddeg_ensemble": np.float64,
+            "clouds_ensemble": np.float64,
+            "radiation_ensemble": np.float64,
+            "ensemble_run": str,
+            "source": str,
+            "created": np.int64,
+        }
+
+        self._write_weather_forecast_data_latest(
+            data=data,
+            source=source,
+            forecast_created_time=forecast_created_time,
+            table=table,
+            dbname=dbname,
+            tag_columns=tag_columns,
+            casting_dict=casting_dict,
+        )
+        self._write_weather_forecast_data_t_ahead(
+            data=data,
+            source=source,
+            forecast_created_time=forecast_created_time,
+            table=table + "_tAhead",
+            dbname=dbname,
+            desired_t_aheads=desired_t_aheads,
+            casting_dict=casting_dict,
+            tag_columns=tag_columns,
+        )
 
     def write_realised(self, df: pd.DataFrame, sid: str):
         """Method that writes measurement data to the influx database.
@@ -338,17 +561,25 @@ class Write:
         # Get rid of last comma
         values = values[0:-1]
 
+        db_type = _DataInterface.get_instance().get_sql_db_type()
+
         # Compose query for writing new systems
-        query = "INSERT IGNORE INTO `systems` (sid, region) VALUES " + values
+        if db_type == "mysql":
+            # Compose query for writing new systems in MySQL
+            query = "INSERT IGNORE INTO `systems` (sid, region) VALUES " + values
+        elif db_type == "postgresql":
+            # Compose query for writing new systems in PostgreSQL
+            query = (
+                "INSERT INTO systems (sid, region) VALUES "
+                + values
+                + " ON CONFLICT (sid) DO NOTHING"
+            )
+        else:
+            self.logger.error("Unsupported database type: {}".format(db_type))
+            return
 
         # Execute query
-        response = _DataInterface.get_instance().mysql_engine.execute(query)
-
-        self.logger.info(
-            "Added {} new systems to the systems table in the MySQL database".format(
-                response.rowcount
-            )
-        )
+        _DataInterface.get_instance().exec_sql_write(query)
 
     def write_kpi(self, pj: dict, kpis: dict) -> None:
         """Method that writes the key performance indicators of a pid to an influx DB.
