@@ -1,12 +1,13 @@
 # SPDX-FileCopyrightText: 2017-2022 Contributors to the OpenSTEF project <korte.termijn.prognoses@alliander.com>
 #
 # SPDX-License-Identifier: MPL-2.0
-import datetime
+from datetime import datetime, timezone
 
+import numpy as np
 import pandas as pd
 
 DEFAULT_FREQ = "15min"
-DEFAULT_TZ = datetime.timezone.utc
+DEFAULT_TZ = timezone.utc
 
 
 def genereate_datetime_index(start, end, freq=None):
@@ -49,7 +50,7 @@ def parse_influx_result(
     if aditional_indices is not None:
         indices.extend(aditional_indices)
 
-    result["_time"] = pd.to_datetime(result["_time"])
+    result.loc[:, "_time"] = pd.to_datetime(result["_time"])
     result = result.pivot_table(
         columns="_field", values="_value", index=indices, aggfunc=aggfunc
     )
@@ -57,3 +58,29 @@ def parse_influx_result(
     result.index.name = "datetime"
     result.columns.name = ""
     return result
+
+
+def _round_down_single_time_diff(time_diff, options):
+    """Rounds a time difference to the first smaller option.
+    Used to calculate the tAhead for forecasts.
+
+    Options should be sorted"""
+
+    next_smaller_option = next((opt for opt in options if opt < time_diff), None)
+    return next_smaller_option
+
+
+def round_down_time_differences(time_diffs, options):
+    """Round a number of time diffs to the first smaller option"""
+    sorted_options = np.sort(options)[::-1]
+    rounded_times = [
+        _round_down_single_time_diff(time_diff, sorted_options)
+        for time_diff in time_diffs
+    ]
+    return rounded_times
+
+
+def floor_to_closest_time_resolution(time: datetime, resolution_minutes: int):
+    return time.replace(
+        minute=time.minute - time.minute % resolution_minutes, second=0, microsecond=0
+    )
